-- Filename:     herk110.vhd
-- Filetype:     VHDL code (behavior)
-- Date:         18 may 2011
-- Update:       12 may 2015
-- Description:  A behavioral description of pattern recognizer
-- Author:       J. op den Brouw
-- State:        demo
-- Error:        -
-- Version:      1.1
-- Copyright:    (c)2015, De Haagse Hogeschool
--
-- This is an example of a behavioral description of a pattern
-- recognizer written as a finite state machine (FSM). It
-- recognizes the pattern 110 continuously while data bits are
-- shifted in sequentially. The output will be logic 1 for one
-- clock cycle following the trailing 0 on the input. The
-- finite state machine is of type Moore.

-- Preamble stuff
library ieee;
use ieee.std_logic_1164.all;

-- The entity
entity herk110 is
    port (clk    : in std_logic;   -- clock input
          areset : in std_logic;   -- asynchronous reset
			 x      : in std_logic;   -- the input of the FSM
          z      : out std_logic); -- the output of the FSM
end entity herk110;

architecture fsm of herk110 is
-- The FSM is constructed with four self named states.
type state_type is (s0, s1, s2, s3);
attribute ENUM_ENCODING : string ;
--attribute ENUM_ENCODING of state_type : type is "11 10 00 01";
--attribute ENUM_ENCODING of state_type : type is "00 01 10 11";
--attribute ENUM_ENCODING of state_type : type is "100 010 110 011";
attribute ENUM_ENCODING of state_type : type is "0001 0010 0100 1000";
signal current_state, next_state : state_type;
begin
    -- The combinational process generates the next state
	 -- and output as a function of the current state and
	 -- the input.
    comb: process (current_state, x) is
	 begin
        -- Default values for next state and output
	     z <= '0';
		  next_state <= s0;
		  -- For every state, describe the next state and the output
        case current_state is
		      when s0 => if x = '1' then
                           next_state <= s1;
                       end if;
		      when s1 => if x = '1' then
                           next_state <= s2;
                       end if;
		      when s2 => if x = '0' then
                           next_state <= s3;
                       else
                           next_state <= s2;
                       end if;
		      when s3 => if x = '1' then
                           next_state <= s1;
                       end if;
							  z <= '1';
            when others => null;
        end case;
	 end process comb;

    -- The sequential process is used to describe the
	 -- state register including the asynchronous reset
	 reg: process (clk, areset, next_state) is
	 begin
	     if areset = '1' then
		      current_state <= s0;
        elsif rising_edge(clk) then
		      current_state <= next_state;
		  end if;
    end process reg;

end fsm;