-- Filename:     tb_herk110.vhd
-- Filetype:     VHDL testbench code
-- Date:         18 may 2011
-- Update:       12 may 2015
-- Description:  A testbench for the pattern recognizer
-- Author:       J. op den Brouw
-- State:        demo
-- Error:        -
-- Version:      1.1
-- Copyright:    (c)2015, De Haagse Hogeschool
--
-- This is an example of a testbench for the pattern recognizer.
-- The testbench is straight forward: declaration and instantiation
-- of the pattern recognizer, a process for generation of the clock
-- and a process for generation of de data bitstream. The clock
-- process is very simple, it just creates a 50% duty cycle clock.
-- The data process is constructed using an variable length array
-- of (std_logic) bits. The array indices are read one by one and
-- assigned to the input of the FSM. The process waits forever when
-- done.

library ieee;
use ieee.std_logic_1164.all;

-- Empty entity, as usual
entity tb_herk110 is
end tb_herk110;

-- The architecture of the testbench
architecture testbench of tb_herk110 is
-- The constants define the timing parameters for the testbench
constant Tperiod : time := 10 ns;
constant small_delay : time := 2 ns;
-- The signals to be assigned and monitored
signal sim_clk : std_logic;
signal sim_areset : std_logic;
signal sim_x : std_logic;
signal sim_z : std_logic;

-- The Design Under Test (DUT)
component herk110
    port (clk : in std_logic;
          areset : in std_logic;
          x : in std_logic;
          z : out std_logic);
end component;

begin
    -- Instantiation of the DUT, using a port map
    dut : herk110
    port map (clk => sim_clk, areset => sim_areset, x => sim_x, z => sim_z);

    -- The clock generation process
    clockgen: process is
    begin
        sim_clk <= '0';
        wait for Tperiod/2;
        sim_clk <= '1';
        wait for Tperiod/2;
    end process clockgen;

    -- The data generation process
	 datagen: process is
	 -- All the valuations for the DUT as string. Note the construction: a
	 -- constant may be unconstrained, but a variable may not. Thus the
	 -- trick is to define an unconstrained constant array and assign it
	 -- to a constrained variable using the range of the previously assigned
	 -- constant.
	 constant x_inputs_setup : std_logic_vector := "0110110101101001110";
	 variable x_inputs : std_logic_vector(x_inputs_setup'range) := x_inputs_setup;
	 begin
        -- Set input to known value
        sim_x <= '0';

        -- Generation of the asynchronous reset signal.
		  -- Set high first and wait two clock cycles
        sim_areset <= '1';
        wait for 2*Tperiod;
	
		  -- Set low and wait two clock cycles
        sim_areset <= '0';
        wait for 2*Tperiod;

        -- Synchronize on the positive edge of the clock, and wait a bit.
        wait until sim_clk = '1';
        wait for small_delay;

        -- Iterate over all array indices		  
		  for index in x_inputs'range loop  -- loop through all input bits
            sim_x <= x_inputs(index);     -- assign to FSM input
            wait until sim_clk = '1';
            wait for small_delay;
		  end loop;

        wait;
	 end process datagen;
	 
--    datagen: process is
--    begin
--        sim_x <= '0';
--        sim_areset <= '1';
--        wait for 2*Tperiod;
--	
--        sim_areset <= '0';
--        wait for 2*Tperiod;
--        wait until sim_clk = '1';
--        wait for small_delay;
--
--        sim_x <= '0';
--        wait until sim_clk = '1';
--        wait for small_delay;
--
--        sim_x <= '1';
--        wait until sim_clk = '1';
--        wait for small_delay;
--
--        sim_x <= '1';
--        wait until sim_clk = '1';
--        wait for small_delay;
--
--        sim_x <= '1';
--        wait until sim_clk = '1';
--        wait for small_delay;
--
--        sim_x <= '0';
--        wait until sim_clk = '1';
--        wait for small_delay;
--
--        wait;
--end process datagen;

end testbench;
